@@ -33,29 +33,49 @@
 
 #include "initialization.h"
 
-#include "simulation_data.h"
-#include "util/omc_error.h"
-#include "openmodelica.h"
-#include "openmodelica_func.h"
-#include "simulation/solver/model_help.h"
+#include "../../../util/omc_error.h"
+#include "../../../util/omc_file.h"
+#include "../../../openmodelica.h"
+#include "../../../openmodelica_func.h"
+#include "../../../simulation/options.h"
+#include "../model_help.h"
 #if !defined(OMC_MINIMAL_RUNTIME)
-#include "util/read_matlab4.h"
+#include "../../../util/read_matlab4.h"
 #endif
-#include "simulation/solver/events.h"
-#include "simulation/solver/stateset.h"
-#include "meta/meta_modelica.h"
+#include "../events.h"
+#include "../stateset.h"
+#include "../../../meta/meta_modelica.h"
 
-#include "simulation/solver/mixedSystem.h"
-#include "simulation/solver/linearSystem.h"
-#include "simulation/solver/nonlinearSystem.h"
-#include "simulation/solver/delay.h"
-#include "simulation/solver/synchronous.h"
+#if defined(OMC_NUM_MIXED_SYSTEMS) && OMC_NUM_MIXED_SYSTEMS==0
+#define check_mixed_solutions(X,Y) 0
+#else
+#include "../mixedSystem.h"
+#endif
+
+#if defined(OMC_NUM_LINEAR_SYSTEMS) && OMC_NUM_LINEAR_SYSTEMS==0
+#define check_linear_solutions(X,Y) 0
+#define updateStaticDataOfLinearSystems(X,Y)
+#else
+#include "../linearSystem.h"
+#endif
+
+#if defined(OMC_NUM_NONLINEAR_SYSTEMS) && OMC_NUM_NONLINEAR_SYSTEMS==0
+#define check_nonlinear_solutions(X,Y) 0
+#define updateStaticDataOfNonlinearSystems(X,Y)
+#else
+#include "../nonlinearSystem.h"
+#endif
+
+#include "../delay.h"
+#include "../synchronous.h"
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <math.h>
 #include <string.h>
 
+extern int init_lambda_steps = 4;
+
 /*! \fn void dumpInitializationStatus(DATA *data)
  *
  *  \param [in]  [data]
@@ -69,8 +89,8 @@ void dumpInitialSolution(DATA *simData)
   const MODEL_DATA      *mData = simData->modelData;
   const SIMULATION_INFO *sInfo = simData->simulationInfo;
 
-  if (ACTIVE_STREAM(LOG_INIT))
-    printParameters(simData, LOG_INIT);
+  if (ACTIVE_STREAM(LOG_INIT_V))
+    printParameters(simData, LOG_INIT_V);
 
   if (!ACTIVE_STREAM(LOG_SOTI)) return;
   infoStreamPrint(LOG_SOTI, 1, "### SOLUTION OF THE INITIALIZATION ###");
@@ -151,72 +171,139 @@ void dumpInitialSolution(DATA *simData)
   messageClose(LOG_SOTI);
 }
 
-/*! \fn static int symbolic_initialization(DATA *data)
+/*! \fn static int symbolic_initialization(DATA *data, threadData_t *threadData)
  *
  *  \param [ref] [data]
+ *  \param [ref] [threadData]
  *
  *  \author lochel
+ *  \author ptaeuber
  */
-static int symbolic_initialization(DATA *data, threadData_t *threadData, long numLambdaSteps)
+static int symbolic_initialization(DATA *data, threadData_t *threadData)
 {
   TRACE_PUSH
-  long step;
   int retVal;
+  FILE *pFile = NULL;
+  long i;
+  MODEL_DATA *mData = data->modelData;
+  int homotopySupport = 0;
+  int solveWithGlobalHomotopy;
+  int adaptiveGlobal;
+  int kinsol = 0;
 
+#if !defined(OMC_MINIMAL_RUNTIME)
+  kinsol = (data->simulationInfo->nlsMethod == NLS_KINSOL);
+#endif
+
+#if !defined(OMC_NUM_NONLINEAR_SYSTEMS) || OMC_NUM_NONLINEAR_SYSTEMS>0
+  for(i=0; i<mData->nNonLinearSystems; i++) {
+    if (data->simulationInfo->nonlinearSystemData[i].homotopySupport) {
+      homotopySupport = 1;
+      break;
+    }
+  }
+#endif
+  /* useHomotopy=1: global homotopy (equidistant lambda) */
+  if (data->callback->useHomotopy == 1 && omc_flag[FLAG_HOMOTOPY_ON_FIRST_TRY] != 1 && omc_flag[FLAG_NO_HOMOTOPY_ON_FIRST_TRY] != 1) {
+      omc_flag[FLAG_HOMOTOPY_ON_FIRST_TRY] = 1;
+      infoStreamPrint(LOG_INIT_HOMOTOPY, 0, "Model contains homotopy operator: Use adaptive homotopy method to solve initialization problem. "
+                                            "To disable initialization with homotopy operator use \"-noHomotopyOnFirstTry\".");
+  }
+
+  adaptiveGlobal = data->callback->useHomotopy == 2;  /* new global homotopy approach (adaptive lambda) */
+  solveWithGlobalHomotopy = homotopySupport
+                            && ((data->callback->useHomotopy == 1 && init_lambda_steps > 1) || adaptiveGlobal);
+
+#if !defined(OMC_NDELAY_EXPRESSIONS) || OMC_NDELAY_EXPRESSIONS>0
   /* initial sample and delay before initial the system */
   initDelay(data, data->simulationInfo->startTime);
-
+#endif
   /* initialize all relations that are ZeroCrossings */
   storePreValues(data);
   overwriteOldSimulationData(data);
 
-  if (data->callback->useHomotopy && numLambdaSteps > 1)
+  /* If there is no homotopy in the model or local homotopy is activated
+     or homotopy is disabled by runtime flag '-ils=<lambda_steps>',
+     solve WITHOUT HOMOTOPY or LOCAL HOMOTOPY. */
+  if (!solveWithGlobalHomotopy){
+    data->simulationInfo->lambda = 1.0;
+    data->callback->functionInitialEquations(data, threadData);
+
+  /* If there is homotopy in the model and global homotopy is activated
+     and homotopy on first try is deactivated,
+     TRY TO SOLVE WITHOUT HOMOTOPY FIRST.
+     TO-DO: For the adaptive global approach, provide a separate DAE with
+     the original unmanipulated systems for trying without homotopy */
+  } else if (!omc_flag[FLAG_HOMOTOPY_ON_FIRST_TRY]) {
+    /* try */
+#ifndef OMC_EMCC
+  MMC_TRY_INTERNAL(simulationJumpBuffer)
+#endif
+    if (adaptiveGlobal && kinsol) {
+      infoStreamPrint(LOG_INIT_HOMOTOPY, 0, "Automatically set -homotopyOnFirstTry, because trying without homotopy first is not supported for the adaptive global approach in combination with KINSOL.");
+    } else {
+      if (adaptiveGlobal)
+        data->callback->useHomotopy = 1;  /* global homotopy (equidistant lambda) */
+      data->simulationInfo->lambda = 1.0;
+      infoStreamPrint(LOG_INIT_HOMOTOPY, 0, "Try to solve the initialization problem without homotopy first.");
+      data->callback->functionInitialEquations(data, threadData);
+      solveWithGlobalHomotopy = 0;
+    }
+
+    /* catch */
+#ifndef OMC_EMCC
+  MMC_CATCH_INTERNAL(simulationJumpBuffer)
+#endif
+    if (adaptiveGlobal)
+      data->callback->useHomotopy = 2; /* new global homotopy approach (adaptive lambda) */
+    if(solveWithGlobalHomotopy) {
+      if (!kinsol)
+        warningStreamPrint(LOG_ASSERT, 0, "Failed to solve the initialization problem without homotopy method. If homotopy is available the homotopy method is used now.");
+      omc_flag[FLAG_HOMOTOPY_ON_FIRST_TRY] = 1;
+      setAllParamsToStart(data);
+      setAllVarsToStart(data);
+      data->callback->updateBoundParameters(data, threadData);
+      data->callback->updateBoundVariableAttributes(data, threadData);
+    }
+  }
+
+  /* If there is homotopy in the model and the equidistant global homotopy approach is activated
+     and solving without homotopy failed or is not wanted,
+     use EQUIDISTANT GLOBAL HOMOTOPY METHOD. */
+  if (data->callback->useHomotopy == 1 && solveWithGlobalHomotopy)
   {
-    long i;
+    long step;
     char buffer[4096];
-    FILE *pFile = NULL;
+    double lambda;
 
-    modelica_real* realVars = (modelica_real*)calloc(data->modelData->nVariablesReal, sizeof(modelica_real));
-    modelica_integer* integerVars = (modelica_integer*)calloc(data->modelData->nVariablesInteger, sizeof(modelica_integer));
-    modelica_boolean* booleanVars = (modelica_boolean*)calloc(data->modelData->nVariablesBoolean, sizeof(modelica_boolean));
-    modelica_string* stringVars = (modelica_string*) omc_alloc_interface.malloc_uncollectable(data->modelData->nVariablesString * sizeof(modelica_string));
-    MODEL_DATA *mData = data->modelData;
+    infoStreamPrint(LOG_INIT_HOMOTOPY, 0, "Global homotopy with equidistant step size started.");
 
-    assertStreamPrint(threadData, 0 != realVars, "out of memory");
-    assertStreamPrint(threadData, 0 != integerVars, "out of memory");
-    assertStreamPrint(threadData, 0 != booleanVars, "out of memory");
-    assertStreamPrint(threadData, 0 != stringVars, "out of memory");
-
-    for(i=0; i<mData->nVariablesReal; ++i) {
-      realVars[i] = mData->realVarsData[i].attribute.start;
-    }
-    for(i=0; i<mData->nVariablesInteger; ++i) {
-      integerVars[i] = mData->integerVarsData[i].attribute.start;
-    }
-    for(i=0; i<mData->nVariablesBoolean; ++i) {
-      booleanVars[i] = mData->booleanVarsData[i].attribute.start;
-    }
-    for(i=0; i<mData->nVariablesString; ++i) {
-      stringVars[i] = mData->stringVarsData[i].attribute.start;
-    }
-
-    if(ACTIVE_STREAM(LOG_INIT))
+#if !defined(OMC_NO_FILESYSTEM)
+    const char sep[] = ",";
+    if(ACTIVE_STREAM(LOG_INIT_HOMOTOPY))
     {
-      sprintf(buffer, "%s_homotopy.csv", mData->modelFilePrefix);
-      pFile = fopen(buffer, "wt");
-      fprintf(pFile, "%s,", "lambda");
-      for(i=0; i<mData->nVariablesReal; ++i)
-        fprintf(pFile, "%s,", mData->realVarsData[i].info.name);
+      sprintf(buffer, "%s_equidistant_global_homotopy.csv", mData->modelFilePrefix);
+      infoStreamPrint(LOG_INIT_HOMOTOPY, 0, "The homotopy path will be exported to %s.", buffer);
+      pFile = omc_fopen(buffer, "wt");
+      fprintf(pFile, "\"lambda\"");
+      for(i=0; i<mData->nVariablesReal; ++i) {
+        fprintf(pFile, "%s\"%s\"", sep, mData->realVarsData[i].info.name);
+      }
       fprintf(pFile, "\n");
     }
+#endif
 
-    infoStreamPrint(LOG_INIT, 1, "homotopy process");
-    for(step=0; step<numLambdaSteps; ++step)
+    infoStreamPrint(LOG_INIT_HOMOTOPY, 1, "homotopy process\n---------------------------");
+    for(step=0; step<init_lambda_steps; ++step)
     {
-      data->simulationInfo->lambda = ((double)step)/(numLambdaSteps-1);
+      data->simulationInfo->lambda = ((double)step)/(init_lambda_steps-1);
+      lambda = data->simulationInfo->lambda;
+      infoStreamPrint(LOG_INIT_HOMOTOPY, 0, "homotopy parameter lambda = %g", lambda);
 
-      if(data->simulationInfo->lambda > 1.0) {
+      if(data->simulationInfo->lambda > 1.0)
+      {
         data->simulationInfo->lambda = 1.0;
+        lambda = 1.0;
       }
 
       if(0 == step)
@@ -224,47 +311,49 @@ static int symbolic_initialization(DATA *data, threadData_t *threadData, long nu
       else
         data->callback->functionInitialEquations(data, threadData);
 
-      infoStreamPrint(LOG_INIT, 0, "lambda = %g done", data->simulationInfo->lambda);
+      infoStreamPrint(LOG_INIT_HOMOTOPY, 0, "homotopy parameter lambda = %g done\n---------------------------", lambda);
 
-      if(ACTIVE_STREAM(LOG_INIT))
+#if !defined(OMC_NO_FILESYSTEM)
+      if(ACTIVE_STREAM(LOG_INIT_HOMOTOPY))
       {
-        fprintf(pFile, "%.16g,", data->simulationInfo->lambda);
+        fprintf(pFile, "%.16g", lambda);
         for(i=0; i<mData->nVariablesReal; ++i)
-          fprintf(pFile, "%.16g,", data->localData[0]->realVars[i]);
+        {
+          fprintf(pFile, "%s%.16g", sep, data->localData[0]->realVars[i]);
+        }
         fprintf(pFile, "\n");
       }
-
-      if(check_nonlinear_solutions(data, 0) ||
-         check_linear_solutions(data, 0) ||
-         check_mixed_solutions(data, 0))
-        break;
-
-      setAllStartToVars(data);
+#endif
     }
-    messageClose(LOG_INIT);
+    data->simulationInfo->homotopySteps += init_lambda_steps;
+    messageClose(LOG_INIT_HOMOTOPY);
 
-    if(ACTIVE_STREAM(LOG_INIT))
+#if !defined(OMC_NO_FILESYSTEM)
+    if(ACTIVE_STREAM(LOG_INIT_HOMOTOPY))
       fclose(pFile);
-
-    for(i=0; i<mData->nVariablesReal; ++i)
-      mData->realVarsData[i].attribute.start = realVars[i];
-    for(i=0; i<mData->nVariablesInteger; ++i)
-      mData->integerVarsData[i].attribute.start = integerVars[i];
-    for(i=0; i<mData->nVariablesBoolean; ++i)
-      mData->booleanVarsData[i].attribute.start = booleanVars[i];
-    for(i=0; i<mData->nVariablesString; ++i)
-      mData->stringVarsData[i].attribute.start = stringVars[i];
-
-    free(realVars);
-    free(integerVars);
-    free(booleanVars);
-    omc_alloc_interface.free_uncollectable(stringVars);
+#endif
   }
-  else
+
+  /* If there is homotopy in the model and the adaptive global homotopy approach is activated
+     and solving without homotopy failed or is not wanted,
+     use ADAPTIVE GLOBAL HOMOTOPY APPROACH. */
+  if (adaptiveGlobal && solveWithGlobalHomotopy)
   {
-    data->simulationInfo->lambda = 1.0;
+    infoStreamPrint(LOG_INIT_HOMOTOPY, 0, "Global homotopy with adaptive step size started.");
+    infoStreamPrint(LOG_INIT_HOMOTOPY, 1, "homotopy process\n---------------------------");
+
+    // Solve lambda0-DAE
+    data->simulationInfo->lambda = 0;
+    infoStreamPrint(LOG_INIT_HOMOTOPY, 0, "solve simplified lambda0-DAE");
+    data->callback->functionInitialEquations_lambda0(data, threadData);
+    infoStreamPrint(LOG_INIT_HOMOTOPY, 0, "solving simplified lambda0-DAE done\n---------------------------");
+
+    // Run along the homotopy path and solve the actual system
     data->callback->functionInitialEquations(data, threadData);
+
+    messageClose(LOG_INIT_HOMOTOPY);
   }
+
   storeRelations(data);
 
   /* check for over-determined systems */
@@ -366,13 +455,10 @@ int importStartValues(DATA *data, threadData_t *threadData, const char *pInitFil
   }
 
   pError = omc_new_matlab4_reader(pInitFile, &reader);
-  if(pError)
-  {
+  if(pError) {
     throwStreamPrint(threadData, "unable to read input-file <%s> [%s]", pInitFile, pError);
     return 1;
-  }
-  else
-  {
+  } else {
     infoStreamPrint(LOG_INIT, 0, "import real variables");
     for(i=0; i<mData->nVariablesReal; ++i) {
       pVar = omc_matlab4_find_var(&reader, mData->realVarsData[i].info.name);
@@ -382,10 +468,9 @@ int importStartValues(DATA *data, threadData_t *threadData, const char *pInitFil
         pVar = omc_matlab4_find_var(&reader, newVarname);
         free(newVarname);
       }
-
       if(pVar) {
         omc_matlab4_val(&(mData->realVarsData[i].attribute.start), &reader, pVar, initTime);
-        infoStreamPrint(LOG_INIT, 0, "| %s(start=%g)", mData->realVarsData[i].info.name, mData->realVarsData[i].attribute.start);
+        infoStreamPrint(LOG_INIT_V, 0, "| %s(start=%g)", mData->realVarsData[i].info.name, mData->realVarsData[i].attribute.start);
       } else if((strlen(mData->realVarsData[i].info.name) > 0) &&
               (mData->realVarsData[i].info.name[0] != '$') &&
               (strncmp(mData->realVarsData[i].info.name, "der($", 5) != 0)) {
@@ -407,7 +492,7 @@ int importStartValues(DATA *data, threadData_t *threadData, const char *pInitFil
       if(pVar) {
         omc_matlab4_val(&(mData->realParameterData[i].attribute.start), &reader, pVar, initTime);
         data->simulationInfo->realParameter[i] = mData->realParameterData[i].attribute.start;
-        infoStreamPrint(LOG_INIT, 0, "| %s(start=%g)", mData->realParameterData[i].info.name, mData->realParameterData[i].attribute.start);
+        infoStreamPrint(LOG_INIT_V, 0, "| %s(start=%g)", mData->realParameterData[i].info.name, mData->realParameterData[i].attribute.start);
       } else {
         warningStreamPrint(LOG_INIT, 0, "unable to import real parameter %s from given file", mData->realParameterData[i].info.name);
       }
@@ -425,7 +510,7 @@ int importStartValues(DATA *data, threadData_t *threadData, const char *pInitFil
 
       if(pVar) {
         omc_matlab4_val(&(mData->realParameterData[i].attribute.start), &reader, pVar, initTime);
-        infoStreamPrint(LOG_INIT, 0, "| %s(start=%g)", mData->realParameterData[i].info.name, mData->realParameterData[i].attribute.start);
+        infoStreamPrint(LOG_INIT_V, 0, "| %s(start=%g)", mData->realParameterData[i].info.name, mData->realParameterData[i].attribute.start);
       } else {
         warningStreamPrint(LOG_INIT, 0, "unable to import real parameter %s from given file", mData->realParameterData[i].info.name);
       }
@@ -446,7 +531,7 @@ int importStartValues(DATA *data, threadData_t *threadData, const char *pInitFil
         omc_matlab4_val(&value, &reader, pVar, initTime);
         mData->integerParameterData[i].attribute.start = (modelica_integer)value;
         data->simulationInfo->integerParameter[i] = (modelica_integer)value;
-        infoStreamPrint(LOG_INIT, 0, "| %s(start=%ld)", mData->integerParameterData[i].info.name, mData->integerParameterData[i].attribute.start);
+        infoStreamPrint(LOG_INIT_V, 0, "| %s(start=%ld)", mData->integerParameterData[i].info.name, mData->integerParameterData[i].attribute.start);
       } else {
         warningStreamPrint(LOG_INIT, 0, "unable to import integer parameter %s from given file", mData->integerParameterData[i].info.name);
       }
@@ -466,7 +551,7 @@ int importStartValues(DATA *data, threadData_t *threadData, const char *pInitFil
         omc_matlab4_val(&value, &reader, pVar, initTime);
         mData->booleanParameterData[i].attribute.start = (modelica_boolean)value;
         data->simulationInfo->booleanParameter[i] = (modelica_boolean)value;
-        infoStreamPrint(LOG_INIT, 0, "| %s(start=%s)", mData->booleanParameterData[i].info.name, mData->booleanParameterData[i].attribute.start ? "true" : "false");
+        infoStreamPrint(LOG_INIT_V, 0, "| %s(start=%s)", mData->booleanParameterData[i].info.name, mData->booleanParameterData[i].attribute.start ? "true" : "false");
       } else {
         warningStreamPrint(LOG_INIT, 0, "unable to import boolean parameter %s from given file", mData->booleanParameterData[i].info.name);
       }
@@ -492,7 +577,7 @@ void initSample(DATA* data, threadData_t *threadData, double startTime, double s
   long i;
 
   data->callback->function_initSample(data, threadData);              /* set-up sample */
-  data->simulationInfo->nextSampleEvent = stopTime + 1.0;  /* should never be reached */
+  data->simulationInfo->nextSampleEvent = NAN;  /* should never be reached */
   for(i=0; i<data->modelData->nSamples; ++i) {
     if(startTime < data->modelData->samplesInfo[i].start) {
       data->simulationInfo->nextSampleTimes[i] = data->modelData->samplesInfo[i].start;
@@ -523,16 +608,19 @@ void initSample(DATA* data, threadData_t *threadData, double startTime, double s
  *
  *  \author lochel
  */
-int initialization(DATA *data, threadData_t *threadData, const char* pInitMethod, const char* pInitFile, double initTime, int lambda_steps)
+int initialization(DATA *data, threadData_t *threadData, const char* pInitMethod, const char* pInitFile, double initTime)
 {
   TRACE_PUSH
   int initMethod = IIM_SYMBOLIC; /* default method */
   int retVal = -1;
   int i;
 
+  data->simulationInfo->homotopySteps = 0;
+
   infoStreamPrint(LOG_INIT, 0, "### START INITIALIZATION ###");
 
-  setAllParamsToStart(data);
+  if (strcmp(pInitMethod, "fmi"))
+    setAllParamsToStart(data);
 
 #if !defined(OMC_MINIMAL_RUNTIME)
   /* import start values from extern mat-file */
@@ -548,12 +636,12 @@ int initialization(DATA *data, threadData_t *threadData, const char* pInitMethod
   }
 #endif
   /* set up all variables with their start-values */
-  setAllVarsToStart(data);
+  if (strcmp(pInitMethod, "fmi"))
+    setAllVarsToStart(data);
 
   if(!(pInitFile && strcmp(pInitFile, ""))) {
     data->callback->updateBoundParameters(data, threadData);
     data->callback->updateBoundVariableAttributes(data, threadData);
-    setAllVarsToStart(data);
   }
 
   /* update static data of linear/non-linear system solvers */
@@ -561,7 +649,7 @@ int initialization(DATA *data, threadData_t *threadData, const char* pInitMethod
   updateStaticDataOfNonlinearSystems(data, threadData);
 
   /* if there are user-specified options, use them! */
-  if (pInitMethod && strcmp(pInitMethod, "")) {
+  if (pInitMethod && (strcmp(pInitMethod, "") && strcmp(pInitMethod, "fmi"))) {
     initMethod = IIM_UNKNOWN;
 
     for (i=1; i<IIM_MAX; ++i) {
@@ -588,34 +676,32 @@ int initialization(DATA *data, threadData_t *threadData, const char* pInitMethod
   /* initialize all (nonlinear|linear|mixed) systems
    * This is a workaround and should be removed as soon as possible.
    */
+#if !defined(OMC_NUM_NONLINEAR_SYSTEMS) || OMC_NUM_NONLINEAR_SYSTEMS>0
   for(i=0; i<data->modelData->nNonLinearSystems; ++i) {
     data->simulationInfo->nonlinearSystemData[i].solved = 1;
   }
+#endif
+#if !defined(OMC_NUM_LINEAR_SYSTEMS) || OMC_NUM_LINEAR_SYSTEMS>0
   for(i=0; i<data->modelData->nLinearSystems; ++i) {
     data->simulationInfo->linearSystemData[i].solved = 1;
   }
+#endif
+#if !defined(OMC_NUM_MIXED_SYSTEMS) || OMC_NUM_MIXED_SYSTEMS>0
   for(i=0; i<data->modelData->nMixedSystems; ++i) {
     data->simulationInfo->mixedSystemData[i].solved = 1;
   }
+#endif
   /* end workaround */
 
   /* select the right initialization-method */
   if(IIM_NONE == initMethod) {
     retVal = 0;
   } else if(IIM_SYMBOLIC == initMethod) {
-    retVal = symbolic_initialization(data, threadData, lambda_steps);
+    retVal = symbolic_initialization(data, threadData);
   } else {
     throwStreamPrint(threadData, "unsupported option -iim");
   }
 
-  /* do pivoting for dynamic state selection if selection changed try again */
-  if(stateSelection(data, threadData, 0, 1) == 1) {
-    if(stateSelection(data, threadData, 1, 1) == 1) {
-      /* report a warning about strange start values */
-      warningStreamPrint(LOG_STDOUT, 0, "Cannot initialize the dynamic state selection in an unique way. Use -lv LOG_DSS to see the switching state set.");
-    }
-  }
-
   /* check for unsolved (nonlinear|linear|mixed) systems
    * This is a workaround and should be removed as soon as possible.
    */
@@ -628,14 +714,26 @@ int initialization(DATA *data, threadData_t *threadData, const char* pInitMethod
   }
   /* end workaround */
 
+#if !defined(OMC_MINIMAL_LOGGING)
   dumpInitialSolution(data);
   infoStreamPrint(LOG_INIT, 0, "### END INITIALIZATION ###");
+#endif
 
-  overwriteOldSimulationData(data);     /* overwrite the whole ring-buffer with initialized values */
-  storePreValues(data);                 /* save pre-values */
-  updateDiscreteSystem(data, threadData);           /* evaluate discrete variables (event iteration) */
+  overwriteOldSimulationData(data);       /* overwrite the whole ring-buffer with initialized values */
+  storePreValues(data);                   /* save pre-values */
+  updateDiscreteSystem(data, threadData); /* evaluate discrete variables (event iteration) */
   saveZeroCrossings(data, threadData);
 
+  /* do pivoting for dynamic state selection if selection changed try again */
+#if !defined(OMC_NO_STATESELECTION)
+  if(stateSelection(data, threadData, 0, 1) == 1) {
+    if(stateSelection(data, threadData, 1, 1) == 1) {
+      /* report a warning about strange start values */
+      warningStreamPrint(LOG_STDOUT, 0, "Cannot initialize the dynamic state selection in an unique way. Use -lv LOG_DSS to see the switching state set.");
+    }
+  }
+#endif
+
   data->simulationInfo->initial = 0;
   /* initialization is done */
 
@@ -644,8 +742,13 @@ int initialization(DATA *data, threadData_t *threadData, const char* pInitMethod
   data->callback->function_updateRelations(data, threadData, 1);
   initSynchronous(data, threadData, data->simulationInfo->startTime);
 
+#if !defined(OMC_MINIMAL_LOGGING)
   printRelations(data, LOG_EVENTS);
   printZeroCrossings(data, LOG_EVENTS);
+#endif
+
+  /* Check for warning of variables out of range assert(min<x || x>xmax, ...)*/
+  data->callback->checkForAsserts(data, threadData);
 
   /* valid system for the first time! */
   TRACE_POP
